import 'dart:convert';
import 'dart:math';

import 'package:shared_preferences/shared_preferences.dart';

class BdConfig {
  String? appId;
  String? roomId;
  String? userId;
  String? userName;
  String? token;
  String? signalUrl;
  bool? autoPublish;
  bool? autoSubscribe;
  bool? enableBdAGC;
  bool? enableBdAEC;
  bool? enableBdNS;
  String? resolution;
  int? fps;
  String? codecType;
  int? bitrate;

  BdConfig(
      {this.appId = "appjfkryqbzyns5",
      this.roomId = "99999999",
      this.userId,
      this.userName,
      this.token = "000000",
      this.signalUrl = "wss://rtc2.exp.bcelive.com/janus",
      this.autoPublish = true,
      this.autoSubscribe = true,
      this.enableBdAGC,
      this.enableBdAEC,
      this.enableBdNS,
      this.resolution,
      this.fps,
      this.codecType,
      this.bitrate});

  static BdConfig fromJson(Map<String, dynamic> jsonMap) {
    BdConfig bdConfig = new BdConfig();
    bdConfig.appId = jsonMap["appId"];
    bdConfig.roomId = jsonMap["roomId"];
    bdConfig.userId = jsonMap["userId"];
    bdConfig.userName = jsonMap["userName"];
    bdConfig.token = jsonMap["token"];
    bdConfig.signalUrl = jsonMap["signalUrl"];
    bdConfig.autoPublish = jsonMap["autoPublish"];
    bdConfig.autoSubscribe = jsonMap["autoSubscribe"];
    bdConfig.enableBdAGC = jsonMap["enableBdAGC"];
    bdConfig.enableBdAEC = jsonMap["enableBdAEC"];
    bdConfig.enableBdNS = jsonMap["enableBdNS"];
    bdConfig.resolution = jsonMap["resolution"];
    bdConfig.fps = jsonMap["fps"];
    bdConfig.codecType = jsonMap["codecType"];
    bdConfig.bitrate = jsonMap["bitrate"];
    return bdConfig;
  }

  Map<String, dynamic> toJson() {
    final Map<String, dynamic> data = new Map<String, dynamic>();
    data['appId'] = this.appId;
    data['roomId'] = this.roomId;
    data['userId'] = this.userId;
    data['userName'] = this.userName;
    data['token'] = this.token;
    data['signalUrl'] = this.signalUrl;
    data['autoPublish'] = this.autoPublish;
    data['autoSubscribe'] = this.autoSubscribe;
    data['enableBdAGC'] = this.enableBdAGC;
    data['enableBdAEC'] = this.enableBdAEC;
    data['enableBdNS'] = this.enableBdNS;
    data['resolution'] = this.resolution;
    data['fps'] = this.fps;
    data['codecType'] = this.codecType;
    data['bitrate'] = this.bitrate;
    return data;
  }
}

class BdUtils {
  static BdConfig? bdConfig;
  static String USERNAME = "Baidu";

  static void initConfig() async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    String? value = prefs.getString("BRTC_CONFIG");
    if (value != null) {
      Map<String, dynamic> jsonMap = json.decode(value);
      bdConfig = BdConfig.fromJson(jsonMap);
    }
    getConfig();
  }

  static void saveConfig(String config) async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    prefs.setString("BRTC_CONFIG", config);
  }

  static BdConfig? getConfig() {
    if (bdConfig == null) {
      bdConfig = new BdConfig();
    }
    return bdConfig;
  }

  static String getSignalServerUrl() {
    if (getConfig()!.signalUrl == null) {
      bdConfig!.signalUrl = "wss://rtc2.exp.bcelive.com/janus";
    }
    return bdConfig!.signalUrl!;
  }

  static String getAppId() {
    if (getConfig()!.appId == null) {
      bdConfig!.appId = "appjfkryqbzyns5";
    }
    return bdConfig!.appId!;
  }

  static String getRoomId() {
    if (getConfig()!.roomId == null) {
      bdConfig!.roomId = "99999999";
    }
    return bdConfig!.roomId!;
  }

  static String getUserId() {
    if (getConfig()!.userId == null) {
      bdConfig!.userId = generateRandomUserId();
    }
    return bdConfig!.userId!;
  }

  static String getUserName() {
    if (getConfig()!.userName == null) {
      bdConfig!.userName = USERNAME + getUserId();
    }
    return bdConfig!.userName!;
  }

  static String getToken() {
    if (getConfig()!.token == null) {
      bdConfig!.token = "000000";
    }
    return bdConfig!.token!;
  }

  static bool getAutoPublishConfig() {
    if (getConfig()!.autoPublish == null) {
      bdConfig!.autoPublish = true;
    }
    return bdConfig!.autoPublish!;
  }

  static bool getAutoSubscribeConfig() {
    if (getConfig()!.autoSubscribe == null) {
      bdConfig!.autoSubscribe = true;
    }
    return bdConfig!.autoSubscribe!;
  }

  static bool getAudioBdAGCConfig() {
    if (getConfig()!.enableBdAGC == null) {
      bdConfig!.enableBdAGC = false;
    }
    return bdConfig!.enableBdAGC!;
  }

  static bool getAudioBdAECConfig() {
    if (getConfig()!.enableBdAEC == null) {
      bdConfig!.enableBdAEC = false;
    }
    return bdConfig!.enableBdAEC!;
  }

  static bool getAudioBdNSConfig() {
    if (getConfig()!.enableBdNS == null) {
      bdConfig!.enableBdNS = false;
    }
    return bdConfig!.enableBdNS!;
  }

  static String getResolutionConfig() {
    if (getConfig()!.resolution == null) {
      bdConfig!.resolution = "640x480";
    }
    return bdConfig!.resolution!;
  }

  static int getFpsConfig() {
    if (getConfig()!.fps == null) {
      bdConfig!.fps = 25;
    }
    return bdConfig!.fps!;
  }

  static String getCodecTypeConfig() {
    if (getConfig()!.codecType == null) {
      bdConfig!.codecType = "H264";
    }
    return bdConfig!.codecType!;
  }

  static int getBitrateConfig() {
    if (getConfig()!.bitrate == null) {
      bdConfig!.bitrate = 1000;
    }
    return bdConfig!.bitrate!;
  }

  static int getMinBitrateConfig() {
    return 100;
  }

  /************************** set config ****************************/

  static void setSignalServerUrl(String signalUrl) {
    getConfig();
    bdConfig!.signalUrl = signalUrl;
    String jsonStr = json.encode(bdConfig);
    saveConfig(jsonStr);
  }

  static void setAppId(String appId) async {
    getConfig();
    bdConfig!.appId = appId;
    String jsonStr = json.encode(bdConfig);
    saveConfig(jsonStr);
  }

  static void setRoomId(String roomId) async {
    getConfig();
    bdConfig!.roomId = roomId;
    String jsonStr = json.encode(bdConfig);
    saveConfig(jsonStr);
  }

  static void setUserId(String userId) async {
    getConfig();
    bdConfig!.userId = userId;
    String jsonStr = json.encode(bdConfig);
    saveConfig(jsonStr);
  }

  static void setUserName(String userName) async {
    getConfig();
    bdConfig!.userName = userName;
    String jsonStr = json.encode(bdConfig);
    saveConfig(jsonStr);
  }

  static void setToken(String token) async {
    getConfig();
    bdConfig!.token = token;
    String jsonStr = json.encode(bdConfig);
    saveConfig(jsonStr);
  }

  static void setAutoPublishConfig(bool on) async {
    getConfig();
    bdConfig!.autoPublish = on;
    String jsonStr = json.encode(bdConfig);
    saveConfig(jsonStr);
  }

  static void setAutoSubscribeConfig(bool on) async {
    getConfig();
    bdConfig!.autoSubscribe = on;
    String jsonStr = json.encode(bdConfig);
    saveConfig(jsonStr);
  }

  static void setAudioBdAGCConfig(bool on) async {
    getConfig();
    bdConfig!.enableBdAGC = on;
    String jsonStr = json.encode(bdConfig);
    saveConfig(jsonStr);
  }

  static void setAudioBdAECConfig(bool on) async {
    getConfig();
    bdConfig!.enableBdAEC = on;
    String jsonStr = json.encode(bdConfig);
    saveConfig(jsonStr);
  }

  static void setAudioBdNSConfig(bool on) async {
    getConfig();
    bdConfig!.enableBdNS = on;
    String jsonStr = json.encode(bdConfig);
    saveConfig(jsonStr);
  }

  static void setResolutionConfig(String resolution) async {
    getConfig();
    bdConfig!.resolution = resolution;
    String jsonStr = json.encode(bdConfig);
    saveConfig(jsonStr);
  }

  static void setFpsConfig(int fps) async {
    getConfig();
    bdConfig!.fps = fps;
    String jsonStr = json.encode(bdConfig);
    saveConfig(jsonStr);
  }

  static void setCodecTypeConfig(String codecType) async {
    getConfig();
    bdConfig!.codecType = codecType;
    String jsonStr = json.encode(bdConfig);
    saveConfig(jsonStr);
  }

  static void setBitrateConfig(int bitrate) async {
    getConfig();
    bdConfig!.bitrate = bitrate;
    String jsonStr = json.encode(bdConfig);
    saveConfig(jsonStr);
  }

  static String generateRandomUserId() {
    String line = "";
    var rng = new Random();
    for (var i = 0; i < 6; i++) {
      int num = rng.nextInt(10);
      if (num <= 0) num = rng.nextInt(10);
      line += num.toString();
    }
    return line;
  }

  static String generateRandomStrRoomId() {
    String line = "";
    var rng = new Random();
    for (var i = 0; i < 7; i++) {
      int num = rng.nextInt(10);
      if (num <= 0) num = rng.nextInt(10);
      line += num.toString();
    }
    return line;
  }
}
