import 'package:brtc_flutter_demo/settings/SettingsPage.dart';
import 'package:flutter/material.dart';
import 'package:oktoast/oktoast.dart';
import 'package:permission_handler/permission_handler.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';

import 'BdUtils.dart';
import 'route/ExampleData.dart';
import 'route/ExampleDataLayout.dart';

class MainPage extends StatefulWidget {
  const MainPage({Key? key}) : super(key: key);

  @override
  _MainPageState createState() => _MainPageState();
}

class _MainPageState extends State<MainPage> {
  List<ExampleItem> _exampleList = [];

  Widget getSeparatorBuilder(BuildContext context, index) {
    Divider divider = Divider(
      thickness: 0,
      height: 20,
    );
    return divider;
  }

  Widget getItemBuilder(context, index) {
    ExampleItem currentItem = this._exampleList[index];
    if (currentItem.detailPage == null) {
      return Text(
        currentItem.title,
        style: TextStyle(fontWeight: FontWeight.bold),
      );
    }

    GestureDetector itemWidget = GestureDetector(
      child: Container(
        alignment: Alignment(0, 0),
        height: 140,
        margin: EdgeInsets.only(left: 10, right: 10),
        decoration: new BoxDecoration(
          color: currentItem.color,
          borderRadius: BorderRadius.all(Radius.circular(6.0)),
        ),
        child: Row(
          children: <Widget>[
            Padding(
              padding: EdgeInsets.only(top: 0, bottom: 0, left: 20),
              child: Column(
                crossAxisAlignment: CrossAxisAlignment.start,
                mainAxisAlignment: MainAxisAlignment.center,
                children: <Widget>[
                  Text(
                    currentItem.title,
                    style: TextStyle(color: Colors.white, fontSize: 24),
                  ),
                  SizedBox(
                    height: 10,
                  ),
                  Text(
                    currentItem.subTitle.toString(),
                    style: TextStyle(color: Colors.white, fontSize: 16),
                  ),
                ],
              ),
            ),
            Expanded(
              child: Container(
                width: 100,
                height: 100,
                child: Image.asset(currentItem.logoPath!),
              ),
            ),
          ],
        ),
      ),
      onTap: () {
        Navigator.push(
          context,
          new MaterialPageRoute(
            builder: (context) => ExampleDataLayout(
              examplePageData: currentItem,
            ),
          ),
        );
      },
    );
    return itemWidget;
  }

  @override
  void initState() {
    super.initState();
    ExampleData.exampleList.forEach((key, value) {
      List<ExampleItem> ls = value;
      _exampleList.add(ExampleItem(title: key, detailPage: null));
      _exampleList.addAll(ls);
    });
    initPermission();
    initConfig();
  }

  void initPermission() async {
    if (!(await Permission.camera.request().isGranted) ||
        !(await Permission.microphone.request().isGranted)) {
      showToast('需要获取音视频权限才能加入房间');
      checkPermission();
      return;
    }
  }

  void initConfig() {
    BdUtils.initConfig();
  }

  // 判断是否有权限
  void checkPermission() async {
    Map<Permission, PermissionStatus> statuses =
        await [Permission.camera, Permission.microphone].request();

    // final info = statuses[Permission.storage];

    Permission permission = Permission.camera;
    PermissionStatus status = statuses[permission]!;
    bool isCameraPass = false;
    print('检测权限: $status');
    if (status.isGranted) {
      // 权限通过
      isCameraPass = true;
    } else if (status.isDenied) {
      // 权限拒绝， 需要区分IOS和Android，二者不一样
      requestPermission(permission);
    } else if (status.isPermanentlyDenied) {
      // 权限永久拒绝，且不在提示，需要进入设置界面
      openAppSettings();
    } else if (status.isRestricted) {
      // 活动限制（例如，设置了家长///控件，仅在iOS以上受支持。
      openAppSettings();
    } else {
      // 第一次申请
      requestPermission(permission);
    }

    if (isCameraPass) {
      Permission permission = Permission.microphone;
      PermissionStatus status = statuses[permission]!;
      print('检测权限: $status');
      if (status.isGranted) {
        // 权限通过
      } else if (status.isDenied) {
        // 权限拒绝， 需要区分IOS和Android，二者不一样
        requestPermission(permission);
      } else if (status.isPermanentlyDenied) {
        // 权限永久拒绝，且不在提示，需要进入设置界面
        openAppSettings();
      } else if (status.isRestricted) {
        // 活动限制（例如，设置了家长///控件，仅在iOS以上受支持。
        openAppSettings();
      } else {
        // 第一次申请
        requestPermission(permission);
      }
    }
  }

  // 申请权限
  void requestPermission(Permission permission) async {
    PermissionStatus status = await permission.request();
    print('权限状态: $status');
    if (!status.isGranted) {
      openAppSettings();
    }
  }

  startSettingPage() {
    ExampleItem item = ExampleItem(
      title: '设置',
      detailPage: SettingsPage(),
    );
    Navigator.push(
      context,
      new MaterialPageRoute(
        builder: (context) => ExampleDataLayout(
          examplePageData: item,
        ),
      ),
    );
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      resizeToAvoidBottomInset: false,
      appBar: AppBar(
        centerTitle: false,
        title: Text('百度云FlutterRTC'),
        titleTextStyle: TextStyle(color: Colors.white, fontSize: 38.sp),
        elevation: 1,
        backgroundColor: Colors.blueAccent,
        actions: <Widget>[
          IconButton(
              onPressed: () {
                startSettingPage();
              },
              iconSize: 10,
              icon: Image.asset("assets/images/settings/icon_setting.png")),
          Padding(
            padding: EdgeInsets.only(top: 0, bottom: 0, left: 0, right: 10),
          )
        ],
      ),
      backgroundColor: Colors.white,
      body: Column(
        children: [
          Expanded(
            child: Container(
              margin: EdgeInsets.zero,
              padding: EdgeInsets.only(top: 0, bottom: 0, left: 10, right: 10),
              child: ListView.separated(
                separatorBuilder: this.getSeparatorBuilder,
                itemBuilder: this.getItemBuilder,
                itemCount: _exampleList.length,
              ),
            ),
          ),
        ],
      ),
    );
  }
}
