import 'package:brtc_flutter_demo/page/videoCall/SingleVideoCallingPage.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:oktoast/oktoast.dart';

import '../page/videoCall/MultiVideoCallingPage.dart';
import '../BdUtils.dart';
import '../route/ExampleData.dart';
import '../route/ExampleDataLayout.dart';

//  BRTC LoginPage.dart
class LoginPage extends StatefulWidget {
  String? type;

  LoginPage({Key? key, this.type}) : super(key: key);

  @override
  _LoginPageState createState() => _LoginPageState(this.type);
}

class _LoginPageState extends State<LoginPage> {
  String appId = BdUtils.getAppId();
  String roomId = BdUtils.getRoomId();
  String userId = BdUtils.getUserId();
  String userName = BdUtils.getUserName();
  String? type;

  final focusNodeRoomId = FocusNode();
  final focusNodeUserId = FocusNode();
  final focusNodeUserName = FocusNode();

  _LoginPageState(String? type) {
    this.type = type;
  }

  gotoCallPage() {
    if (this.type == ExampleItem.TYPE_SINGLE_VIDEO) {
      startVideoSinglePage();
    } else if (this.type == ExampleItem.TYPE_MULTI_VIDEO) {
      startVideoMultiPage();
    }
  }

  startVideoSinglePage() {
    ExampleItem item = ExampleItem(
      title: '房间号: $roomId',
      detailPage: SingleVideoCallingPage(
        appId: appId,
        roomId: roomId,
        userId: userId,
        userName: userName,
      ),
    );
    Navigator.push(
      context,
      new MaterialPageRoute(
        builder: (context) => ExampleDataLayout(
          examplePageData: item,
        ),
      ),
    );
  }

  startVideoMultiPage() {
    ExampleItem item = ExampleItem(
      title: '房间号: $roomId',
      detailPage: MultiVideoCallingPage(
        appId: appId,
        roomId: roomId,
        userId: userId,
        userName: userName,
      ),
    );
    Navigator.push(
      context,
      new MaterialPageRoute(
        builder: (context) => ExampleDataLayout(
          examplePageData: item,
        ),
      ),
    );
  }

  @override
  void initState() {
    print("[Flutter] LoginPage initState");
    super.initState();
  }

  @override
  void deactivate() {
    print("[Flutter] LoginPage deactivate");
    super.deactivate();
  }

  @override
  void dispose() {
    print("[Flutter] LoginPage dispose");
    cancelFocus();
    super.dispose();
  }

  void cancelFocus() {
    if (focusNodeRoomId.hasFocus) {
      focusNodeRoomId.unfocus();
    } else if (focusNodeUserId.hasFocus) {
      focusNodeUserId.unfocus();
    } else if (focusNodeUserName.hasFocus) {
      focusNodeUserName.unfocus();
    }
  }

  @override
  Widget build(BuildContext context) {
    return GestureDetector(
        behavior: HitTestBehavior.translucent,
        onTap: () {
          cancelFocus();
        },
        child: SingleChildScrollView(
          child: Column(
            children: [
              Expanded(
                flex: 0,
                child: Padding(
                  padding: EdgeInsets.only(top: 40, left: 20, right: 20),
                  child: Column(
                    crossAxisAlignment: CrossAxisAlignment.center,
                    mainAxisAlignment: MainAxisAlignment.center,
                    children: [
                      Text(
                        "欢迎来到百度云RTC",
                        textAlign: TextAlign.center,
                        style: TextStyle(color: Colors.black, fontSize: 24),
                      ),
                      SizedBox(height: 3),
                      Text(
                        "进入房间后开始你的旅程",
                        textAlign: TextAlign.center,
                        style: TextStyle(color: Colors.black, fontSize: 12),
                      ),
                    ],
                  ),
                ),
              ),
              Expanded(
                flex: 0,
                child: Padding(
                  padding: EdgeInsets.only(top: 30, left: 20, right: 20),
                  child: Column(
                    children: [
                      TextField(
                        style: TextStyle(color: Colors.blueAccent),
                        autofocus: false,
                        controller: TextEditingController.fromValue(
                          TextEditingValue(
                            text: this.roomId.toString(),
                            selection: TextSelection.fromPosition(
                              TextPosition(
                                  affinity: TextAffinity.downstream,
                                  offset: '${this.roomId}'.length),
                            ),
                          ),
                        ),
                        focusNode: focusNodeRoomId,
                        decoration: InputDecoration(
                          labelText: "请输入房间号",
                          hintText: "请输入房间号",
                          labelStyle: TextStyle(color: Colors.black),
                          hintStyle:
                              TextStyle(color: Color.fromRGBO(44, 118, 255, 1)),
                          enabledBorder: UnderlineInputBorder(
                            borderSide: BorderSide(color: Colors.black),
                          ),
                        ),
                        keyboardType: TextInputType.number,
                        onChanged: (value) {
                          roomId = value;
                        },
                      ),
                      SizedBox(height: 30),
                      TextField(
                        style: TextStyle(color: Colors.blueAccent),
                        autofocus: false,
                        controller: TextEditingController.fromValue(
                          TextEditingValue(
                            text: this.userId,
                            selection: TextSelection.fromPosition(
                              TextPosition(
                                  affinity: TextAffinity.downstream,
                                  offset: '${this.userId}'.length),
                            ),
                          ),
                        ),
                        focusNode: focusNodeUserId,
                        decoration: InputDecoration(
                          labelText: "请输入用户ID",
                          hintText: "请输入用户ID",
                          labelStyle: TextStyle(color: Colors.black),
                          hintStyle:
                              TextStyle(color: Color.fromRGBO(44, 118, 255, 1)),
                          enabledBorder: UnderlineInputBorder(
                            borderSide: BorderSide(color: Colors.black),
                          ),
                        ),
                        keyboardType: TextInputType.text,
                        onChanged: (value) {
                          this.userId = value;
                        },
                      ),
                      SizedBox(height: 30),
                      TextField(
                        style: TextStyle(color: Colors.blueAccent),
                        autofocus: false,
                        controller: TextEditingController.fromValue(
                          TextEditingValue(
                            text: this.userName,
                            selection: TextSelection.fromPosition(
                              TextPosition(
                                  affinity: TextAffinity.downstream,
                                  offset: '${this.userId}'.length),
                            ),
                          ),
                        ),
                        focusNode: focusNodeUserName,
                        decoration: InputDecoration(
                          labelText: "请输入用户名",
                          hintText: "请输入用户名",
                          labelStyle: TextStyle(color: Colors.black),
                          hintStyle:
                              TextStyle(color: Color.fromRGBO(44, 118, 255, 1)),
                          enabledBorder: UnderlineInputBorder(
                            borderSide: BorderSide(color: Colors.black),
                          ),
                        ),
                        keyboardType: TextInputType.text,
                        onChanged: (value) {
                          this.userName = value;
                        },
                      ),
                    ],
                  ),
                ),
              ),
              Expanded(
                flex: 0,
                child: Padding(
                  padding: EdgeInsets.only(top: 100, bottom: 100),
                  child: ElevatedButton(
                    style: ButtonStyle(
                      minimumSize: MaterialStateProperty.all(Size(320, 40)),
                      backgroundColor:
                          MaterialStateProperty.all(Colors.blueAccent),
                    ),
                    onPressed: () {
                      if (this.roomId.isEmpty ||
                          this.userId.isEmpty ||
                          this.userName.isEmpty) {
                        showToast("信息填写不全");
                        return;
                      }
                      BdUtils.setRoomId(this.roomId);
                      BdUtils.setUserId(this.userId);

                      this.gotoCallPage();
                    },
                    child: Text("开始通话"),
                  ),
                ),
              ),
            ],
          ),
        ));
  }
}
