import 'package:bdcloud_rtc/brtc_cloud.dart';
import 'package:bdcloud_rtc/brtc_room_config.dart';
import 'package:bdcloud_rtc/brtc_room_listener.dart';
import 'package:bdcloud_rtc/brtc_video_view.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:oktoast/oktoast.dart';

import '../../BdUtils.dart';
import '../views/IEventCallback.dart';

// base video call
abstract class BaseCallingPageState implements IEventCallback {
  Map<String, String> remoteUserIdMap = {};
  // 非自动订阅情况下使用，房间内全部远端用户
  Map<String, String> remoteUserIdAllMap = {};
  Map<String, int> remoteUserViewMap = {};
  Map<String, BRTCCloudVideoView> remoteUserIdVideoViewMap = {};
  int subscriberCount = 0;
  bool isFrontCamera = true;
  bool isOpenCamera = true;
  int? localViewId;

  int videoWidth = 480;
  int videoHeight = 640;

  // 是否自动订阅
  bool isAutoPublish = BdUtils.getAutoPublishConfig();

  // 是否自动订阅
  bool isAutoSubscribe = BdUtils.getAutoSubscribeConfig();

  bool isMuteLocalAudio = false;
  bool isSpeaker = true;
  late BaiduRtcCloud baiduRtcCloud;

  void onChangeAppLifecycleState(AppLifecycleState state) {
    // TODO: implement didChangeAppLifecycleState
    switch (state) {
      case AppLifecycleState.inactive:
        // 当前应用闲置状态
        break;
      case AppLifecycleState.paused:
        // 当前应用处于不可见状态
        break;
      case AppLifecycleState.resumed:
        // 当前应用处于可见状态，
        // Android会在一分钟后停止相机权限，Camera device could not be opened due to a device policy.
        break;
      case AppLifecycleState.detached:
        // 当前应用页面即将退出
        break;
      default:
        break;
    }
  }

  void updateVideoWidthHeight(videoResolution) {
    String res = videoResolution;
    if (res.contains("192x144")) {
      videoHeight = 192;
      videoWidth = 144;
    } else if (res.contains("320x180")) {
      videoHeight = 320;
      videoWidth = 180;
    } else if (res.contains("352x288")) {
      videoHeight = 352;
      videoWidth = 288;
    } else if (res.contains("480x320")) {
      videoHeight = 480;
      videoWidth = 320;
    } else if (res.contains("320x480")) {
      videoHeight = 320;
      videoWidth = 480;
    } else if (res.contains("480x360")) {
      videoHeight = 480;
      videoWidth = 360;
    } else if (res.contains("640x360")) {
      videoHeight = 640;
      videoWidth = 360;
    } else if (res.contains("640x480")) {
      videoHeight = 640;
      videoWidth = 480;
    } else if (res.contains("960x540")) {
      videoHeight = 960;
      videoWidth = 540;
    } else if (res.contains("1280x720")) {
      videoHeight = 1280;
      videoWidth = 720;
    } else if (res.contains("1920x1080")) {
      videoHeight = 1920;
      videoWidth = 1080;
    } else if (res.contains("3840x2160")) {
      videoHeight = 3840;
      videoWidth = 2160;
    } else if (res.contains("4096x2160")) {
      videoHeight = 4096;
      videoWidth = 2160;
    } else if (res.contains("7680x4320")) {
      videoHeight = 7680;
      videoWidth = 4320;
    } else if (res.contains("8192x4320")) {
      videoHeight = 8192;
      videoWidth = 4320;
    } else if (res.contains("128x96")) {
      videoHeight = 128;
      videoWidth = 96;
    } else if (res.contains("176x144")) {
      videoHeight = 176;
      videoWidth = 144;
    } else if (res.contains("352x288")) {
      videoHeight = 352;
      videoWidth = 288;
    } else if (res.contains("704x576")) {
      videoHeight = 704;
      videoWidth = 576;
    } else if (res.contains("1408x1152")) {
      videoHeight = 1408;
      videoWidth = 1152;
    } else {
      List<String> list = res.split("\\(")[0].split("x");
      videoHeight = int.parse(list[0]);
      videoWidth = int.parse(list[1]);
    }
    if (videoHeight > 8192 || videoWidth > 4320) {
      videoHeight = 8192;
      videoWidth = 4320;
    }
  }

  void enterRoom(
      String appId, String userId, String roomId, String userName) async {
    // 初始化
    BRTCInitParams initParams = new BRTCInitParams();
    initParams.appId = appId;
    baiduRtcCloud = (await BaiduRtcCloud.getInstance(initParams))!;

    // 设置参数
    BRTCCommonParam commonParam = BRTCCommonParam();
    commonParam.autoPublish = isAutoPublish;
    commonParam.autoSubscribe = isAutoSubscribe;
    commonParam.enableMultistream = true;
    commonParam.videoResolution = BdUtils.getResolutionConfig();
    commonParam.videoMaxkbps = BdUtils.getBitrateConfig();
    commonParam.videoMinkbps = BdUtils.getMinBitrateConfig();
    commonParam.videoFps = BdUtils.getFpsConfig();

    commonParam.connectionTimeoutMs = 5000;
    commonParam.readTimeoutMs = 5000;
    commonParam.inputAudioChannel = 1;
    commonParam.outputAudioChannel = 1;
    commonParam.audioFrequency = 48000;
    commonParam.enableJitterRetransmission = false;
    commonParam.enableFixedResolution = false;
    updateVideoWidthHeight(BdUtils.getResolutionConfig());
    commonParam.videoWidth = videoWidth;
    commonParam.videoHeight = videoHeight;
    commonParam.isMultiPlayerModel = true;
    baiduRtcCloud.setParamSettings(commonParam);

    // 设置信令服务器地址
    baiduRtcCloud.setMediaServerURL(BdUtils.getSignalServerUrl(), "");

    // 音频3A
    baiduRtcCloud.enableBdAEC(BdUtils.getAudioBdAECConfig());
    baiduRtcCloud.enableAec(!BdUtils.getAudioBdAECConfig());
    baiduRtcCloud.enableBdAGC(BdUtils.getAudioBdAGCConfig());
    baiduRtcCloud.enableAgc(!BdUtils.getAudioBdAGCConfig());
    baiduRtcCloud.enableBdANS(BdUtils.getAudioBdNSConfig());
    baiduRtcCloud.enableAns(!BdUtils.getAudioBdNSConfig());

    baiduRtcCloud.presetLoudSpeaker(true);

    baiduRtcCloud.enableErrorInfoToServer(false, "");
    baiduRtcCloud.enableStatsToServer(false, "");

    // 开启log
    baiduRtcCloud.setVerbose(true);

    // 注册回调
    baiduRtcCloud.registerListener(onRtcRoomListener);

    // 登录房间
    BRTCLoginParams loginParams = BRTCLoginParams();
    loginParams.userId = userId;
    loginParams.roomId = roomId;
    loginParams.displayName = userName;
    baiduRtcCloud.loginBRtcRoom(loginParams);
  }

  void printLog(eventCode, msg) {
    print("[Receive onRoomEventUpdate] : eventCode= " +
        eventCode.toString() +
        " msg= " +
        msg);
  }

  void onRtcRoomListener(type, eventCode, params) {
    String msg = "";
    switch (type) {
      case BaiduRTCRoomDelegate.onRoomEventUpdate:
        switch (eventCode) {
          case BRtcEvents.BRTC_ROOM_EVENT_LOGIN_ERROR:
            msg = "Login Room Failed";
            break;
          case BRtcEvents.BRTC_ROOM_EVENT_WS_CONNECTION_ERROR:
            msg = "Connection Lost";
            onExitRoom();
            break;
          case BRtcEvents.BRTC_ROOM_EVENT_ON_USER_JOINED_ROOM:
            // user join
            msg = "用户进入" + params["data"]!.toString();
            remoteUserJoinRoom(params["data"]!.toString());
            break;
          case BRtcEvents.BRTC_ROOM_EVENT_ON_USER_LEAVING_ROOM:
            // user leave
            msg = "用户离开" + params["data"]!.toString();
            remoteUserLeaveRoom(params["data"]!.toString());
            break;
          case BRtcEvents.BRTC_STATE_SENDING_MEDIA_OK:
            msg = "服务端收到媒体流";
            break;
          case BRtcEvents.BRTC_STATE_SENDING_MEDIA_FAILED:
            msg = "服务端未收到媒体流";
            break;
          case BRtcEvents.BRTC_ROOM_EVENT_ON_USER_MESSAGE:
            String extraInfo = params["extra_info"];
            if (extraInfo.contains("ping getattribute")) {
              baiduRtcCloud.getUserAttribute(params["data"]!);
            }
            if (extraInfo.contains("ping sendmessage")) {
              baiduRtcCloud.sendMessageToUser("hello", 0);
            }
            if (extraInfo.contains("ping setattribute")) {
              baiduRtcCloud
                  .setUserAttribute("{'name':'jim','tel':'123456789'}");
            }
            break;
          default:
            break;
        }
        printLog(eventCode.toString(), msg);
        break;
      case BaiduRTCRoomDelegate.onPeerConnectStateUpdate:
        print("[Receive onPeerConnectStateUpdate] : state=  " +
            params['connectState'].toString());
        break;
      case BaiduRTCRoomDelegate.onRoomDataMessage:
        print("[Receive onRoomDataMessage] : message=  " +
            params['message'].toString());
        break;
      case BaiduRTCRoomDelegate.onEngineStatisticsInfo:
        print("[Receive onEngineStatisticsInfo] : handleId=  " +
            params['handleId'].toString());
        break;
      case BaiduRTCRoomDelegate.onStreamInfoUpdate:
        print("[Receive onStreamInfoUpdate] : streamIds=  " +
            params['streamIds'].toString());
        break;
      case BaiduRTCRoomDelegate.onErrorInfoUpdate:
        print("[Receive onErrorInfoUpdate] : errorInfo=  " +
            params['errorInfo'].toString());
        break;
      default:
        break;
    }
  }

  void remoteUserJoinRoom(String userId) {
    if (isAutoSubscribe) {
      onUserJoinRoom(userId);
    }
    remoteUserIdAllMap[userId] = userId;
  }

  void remoteUserLeaveRoom(String userId) {
    if (remoteUserIdAllMap.containsKey(userId)) {
      remoteUserIdAllMap.remove(userId);
    }
    onUserLeaveRoom(userId);
  }

  onUserJoinRoom(String userId);

  onUserLeaveRoom(String userId);

  onExitRoom();

  void destroyRoom() {
    baiduRtcCloud.stopPreview();
    baiduRtcCloud.unRegisterListener(onRtcRoomListener);
    baiduRtcCloud.logoutRtcRoom();
    BaiduRtcCloud.destroyInstance();
  }

  @override
  void onSubscribe(String userId) {
    if (isAutoSubscribe) {
      showToast("自动订阅状态下无法订阅用户");
      return;
    }
    if (baiduRtcCloud != null) {
      printLog("MultiVideo SubscribeUser", userId);
      if (remoteUserIdAllMap.containsKey(userId)) {
        baiduRtcCloud.subscribeStreaming(int.parse(userId));
        onUserJoinRoom(userId);
      } else {
        showToast("订阅用户未在房间内");
      }
    }
  }

  @override
  void onStopSubscribe(String userId) {
    if (isAutoSubscribe) {
      showToast("自动订阅状态下无法取消订阅用户");
      return;
    }
    if (baiduRtcCloud != null) {
      baiduRtcCloud.stopSubscribeStreaming(int.parse(userId));
      onUserLeaveRoom(userId);
    }
  }

  @override
  void onPublish() {
    if (isAutoPublish) {
      showToast("自动发布状态下无法发布");
      return;
    }
    if (baiduRtcCloud != null) {
      baiduRtcCloud.startPublish();
    }
  }
}
