import 'package:bdcloud_rtc/brtc_room_config.dart';
import 'package:bdcloud_rtc/brtc_video_view.dart';
import 'package:brtc_flutter_demo/page/views/TopFunctionView.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';
import 'package:wakelock/wakelock.dart';

import '../views/PhotoTextView.dart';
import 'BaseCallingPage.dart';

// multi person video call
class MultiVideoCallingPage extends StatefulWidget {
  String appId = "";
  String roomId = "";
  String userId = "";
  String userName = "";

  MultiVideoCallingPage(
      {Key? key,
      required this.appId,
      required this.roomId,
      required this.userId,
      required this.userName})
      : super(key: key);

  @override
  _MultiVideoCallingPageState createState() => _MultiVideoCallingPageState();
}

class _MultiVideoCallingPageState extends State<MultiVideoCallingPage>
    with WidgetsBindingObserver, BaseCallingPageState {
  @override
  void initState() {
    enterRoom(this.widget.appId, this.widget.userId, this.widget.roomId,
        this.widget.userName);
    super.initState();
    Wakelock.enable();
    WidgetsBinding.instance!.addObserver(this);
  }

  @override
  void didChangeAppLifecycleState(AppLifecycleState state) {
    super.didChangeAppLifecycleState(state);
    onChangeAppLifecycleState(state);
  }

  @override
  void onUserJoinRoom(String userId) {
    setState(() {
      printLog("MultiVideo UserJoinRoom", userId);
      remoteUserIdMap[userId] = userId;
    });
  }

  @override
  void onUserLeaveRoom(String userId) {
    setState(() {
      if (baiduRtcCloud != null) {
        baiduRtcCloud.removeRemoteDisplay(int.parse(userId));
      }

      printLog("MultiVideo UserLeaveRoom", userId);
      if (remoteUserIdMap.containsKey(userId)) {
        remoteUserIdMap.remove(userId);
        remoteUserViewMap.remove(userId);
      }
    });
  }

  void onExitRoom() {
    Navigator.pop(context);
  }

  @override
  void dispose() {
    print("MultiVideoCallingPage dispose");
  }

  @override
  void deactivate() {
    // TODO: implement deactivate
    super.deactivate();
    print("[Flutter] MultiVideoCallingPage deactivate");
    destroyRoom();
    remoteUserViewMap = {};
    remoteUserIdMap = {};
    super.dispose();
    Wakelock.disable();
    WidgetsBinding.instance!.removeObserver(this);
  }

  Widget? localVideoView;

  Widget getLocalVideoView() {
    return Container(
      child: BRTCCloudVideoView(
        key: ValueKey("LocalView"),
        viewType: BRTCCloudConfig.BRTC_Video_SurfaceView_Channel,
        videoParam: BRTCVideoParams(
          x: 0,
          y: 0,
          width: (1 / 2).sw.toInt(),
          height: (2 / 3).sw.toInt(),
          isLocal: true,
        ),
        onViewCreated: (viewId) async {
          setState(() {
            localViewId = viewId;
          });
          baiduRtcCloud.setLocalDisplay(viewId);
        },
      ),
    );
  }

  Widget getRemoteVideoView(String userId, int index) {
    Widget remoteVideoView;

    remoteVideoView = Stack(
      alignment: Alignment.topLeft,
      fit: StackFit.expand,
      children: <Widget>[
        Container(
          child: BRTCCloudVideoView(
            key: ValueKey("RemoteView_$userId"),
            viewType: BRTCCloudConfig.BRTC_Video_Remote_View_Channel,
            videoParam: BRTCVideoParams(
              x: 0,
              y: 0,
              width: (1 / 2).sw.toInt(),
              height: (2 / 3).sw.toInt(),
              isLocal: false,
            ),
            onViewCreated: (viewId) async {
              if (remoteUserViewMap.containsKey(userId)) {
                baiduRtcCloud.updateDisplay(viewId, int.parse(userId));
              } else {
                baiduRtcCloud.setRemoteDisplayWithUserId(
                    viewId, int.parse(userId));
                remoteUserViewMap[userId] = viewId;
              }
            },
          ),
        ),
        Positioned(
            top: 10,
            left: 10,
            child: Container(
              color: Color(0x66B8BABF),
              padding: EdgeInsets.only(
                  left: 6.sp, top: 6.sp, right: 6.sp, bottom: 6.sp),
              child: Text(
                "ID: $userId",
                style: TextStyle(color: Colors.white, fontSize: 20.sp),
              ),
            ))
      ],
    );

    return remoteVideoView;
  }

  @override
  Widget build(BuildContext context) {
    final double distance = 12;
    List<String> remoteUidList = remoteUserIdMap.values.toList();
    subscriberCount = remoteUidList.length;
    return Stack(
      alignment: Alignment.topLeft,
      fit: StackFit.expand,
      children: [
        Positioned(
          key: ValueKey("Background"),
          top: 0,
          left: 0,
          child: Container(
            width: 1.sw,
            height: 1.sh,
            color: Colors.black,
          ),
        ),
        Positioned(
          top: 10.sp,
          left: 0,
          right: 0,
          child: TopFunctionView(
            height: 80.sp,
            width: 1.sw,
            iEventCallback: this,
          ),
        ),
        Positioned(
          top: 100.sp,
          left: 0,
          right: 0,
          child: Row(
            mainAxisAlignment: MainAxisAlignment.start,
            mainAxisSize: MainAxisSize.min,
            children: [
              Container(
                alignment: Alignment.topCenter,
                width: 1.sw,
                height: ((1.sw /2)* 2 * 4 / 3),
                child: GridView.builder(
                  scrollDirection: Axis.horizontal,
                  itemCount: (subscriberCount + 1),
                  shrinkWrap: false,
                  gridDelegate: SliverGridDelegateWithFixedCrossAxisCount(
                      // 横轴元素个数
                      crossAxisCount: 2,
                      // 纵轴间距
                      mainAxisSpacing: 2.0,
                      // 横轴间距
                      crossAxisSpacing: 5.0,
                      // 子组件宽高长度比例
                      childAspectRatio: (4 / 3.0)),
                  itemBuilder: (BuildContext context, int index) {
                    if (index == 0) {
                      return getLocalVideoView();
                    } else {
                      String userId = index.toString();
                      if (remoteUidList.length > (index - 1)) {
                        userId = remoteUidList[index - 1];
                      }

                      return getRemoteVideoView(userId, index);
                    }
                  },
                ),
              ),
            ],
          ),
        ),
        Positioned(
          left: 6,
          right: 6,
          bottom: 10,
          child: Row(
            mainAxisAlignment: MainAxisAlignment.spaceEvenly,
            children: [
              Row(
                children: [
                  PhotoTextView(
                    key: ValueKey("muteAudio" + isMuteLocalAudio.toString()),
                    text: "静音",
                    imagePath: isMuteLocalAudio
                        ? "assets/images/callfunc/btn_mute_selected.png"
                        : "assets/images/callfunc/btn_mute.png",
                    clickListener: () {
                      bool newIsMuteLocalAudio = !isMuteLocalAudio;
                      baiduRtcCloud.muteMicphone(newIsMuteLocalAudio);
                      setState(() {
                        isMuteLocalAudio = newIsMuteLocalAudio;
                      });
                    },
                  ),
                  SizedBox(
                    width: distance,
                  ),
                  PhotoTextView(
                    key: ValueKey("speaker" + isSpeaker.toString()),
                    text: "扬声器",
                    imagePath: isSpeaker
                        ? "assets/images/callfunc/btn_speaker_selected.png"
                        : "assets/images/callfunc/btn_speaker.png",
                    clickListener: () {
                      bool newIsSpeaker = !isSpeaker;
                      baiduRtcCloud.switchLoundSpeaker();
                      setState(() {
                        isSpeaker = newIsSpeaker;
                      });
                    },
                  ),
                  SizedBox(
                    width: distance,
                  ),
                  PhotoTextView(
                    key: ValueKey("cancel"),
                    text: "挂断",
                    imagePath: "assets/images/callfunc/btn_end_call.png",
                    clickListener: () {
                      // 挂断
                      Navigator.pop(context);
                    },
                  ),
                  SizedBox(
                    width: distance,
                  ),
                  PhotoTextView(
                    key: ValueKey("openCamera" + isOpenCamera.toString()),
                    text: "切到语音",
                    imagePath: !isOpenCamera
                        ? "assets/images/callfunc/btn_voice_selected.png"
                        : "assets/images/callfunc/btn_voice.png",
                    clickListener: () {
                      bool newIsOpenCamera = isOpenCamera;
                      baiduRtcCloud.muteCamera(newIsOpenCamera);
                      setState(() {
                        isOpenCamera = !newIsOpenCamera;
                      });
                    },
                  ),
                  SizedBox(
                    width: distance,
                  ),
                  PhotoTextView(
                    key: ValueKey("frontCamera" + isFrontCamera.toString()),
                    text: "摄像头",
                    imagePath: !isFrontCamera
                        ? "assets/images/callfunc/btn_switch_camera_selected.png"
                        : "assets/images/callfunc/btn_switch_camera.png",
                    clickListener: () {
                      bool newIsFrontCamera = !isFrontCamera;
                      baiduRtcCloud.switchCamera();
                      setState(() {
                        isFrontCamera = newIsFrontCamera;
                      });
                    },
                  ),
                ],
              ),
            ],
          ),
        ),
      ],
    );
  }
}
