import 'package:bdcloud_rtc/brtc_room_config.dart';
import 'package:bdcloud_rtc/brtc_video_view.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';
import 'package:wakelock/wakelock.dart';

import '../views/PhotoTextView.dart';
import 'BaseCallingPage.dart';

// 1v1 video call
class SingleVideoCallingPage extends StatefulWidget {
  String appId = "";
  String roomId = "";
  String userId = "";
  String userName = "";

  SingleVideoCallingPage(
      {Key? key,
      required this.appId,
      required this.roomId,
      required this.userId,
      required this.userName})
      : super(key: key);

  @override
  _SingleVideoCallingPageState createState() => _SingleVideoCallingPageState();
}

class _SingleVideoCallingPageState extends State<SingleVideoCallingPage>
    with WidgetsBindingObserver, BaseCallingPageState {
  @override
  void initState() {
    enterRoom(this.widget.appId, this.widget.userId, this.widget.roomId,
        this.widget.userName);
    super.initState();
    Wakelock.enable();
    WidgetsBinding.instance!.addObserver(this);
  }

  @override
  void didChangeAppLifecycleState(AppLifecycleState state) {
    // TODO: implement didChangeAppLifecycleState
    super.didChangeAppLifecycleState(state);
    onChangeAppLifecycleState(state);
  }

  @override
  void onUserJoinRoom(String userId) {
    setState(() {
      remoteUserIdMap[userId] = userId;
    });
  }

  @override
  void onUserLeaveRoom(String userId) {
    print("[Flutter] SingleVideoCallingPage onUserLeaveRoom userId=" + userId);
    if (remoteUserIdVideoViewMap.containsKey(userId)) {
      if (remoteUserIdVideoViewMap[userId]!.videoViewController != null) {
        remoteUserIdVideoViewMap[userId]!.videoViewController!
            .destroyPlatView();
      }
    }
    remoteUserIdVideoViewMap.remove(userId);
    setState(() {
      print("[Flutter] SingleVideoCallingPage onUserLeaveRoom setState..");
    });
    setState(() {
      if (baiduRtcCloud != null) {
        baiduRtcCloud.removeRemoteDisplay(int.parse(userId));
      }

      if (remoteUserIdMap.containsKey(userId)) {
        remoteUserIdMap.remove(userId);
        remoteUserViewMap.remove(userId);
      }

    });

    setState(() {

    });
  }

  void onExitRoom() {
    Navigator.pop(context);
  }

  @override
  void dispose() {
    print("[Flutter] SingleVideoCallingPage dispose");
  }

  @override
  void deactivate() {
    // TODO: implement deactivate
    super.deactivate();
    print("[Flutter] SingleVideoCallingPage deactivate");
    destroyRoom();
    remoteUserViewMap.clear();
    remoteUserIdMap.clear();
    remoteUserIdVideoViewMap.clear();
    super.dispose();
    Wakelock.disable();
    WidgetsBinding.instance!.removeObserver(this);
  }

  Widget getRemoteView(List<String> remoteUidList, int index) {
    BRTCCloudVideoView remoteView = BRTCCloudVideoView(
      key: ValueKey('RemoteView_$index'),
      viewType: BRTCCloudConfig.BRTC_Video_Remote_View_Channel,
      videoParam: BRTCVideoParams(
        x: 0,
        y: 0,
        width: (1 / 3).sw.toInt(),
        height: (4 / 9).sw.toInt(),
        isLocal: false,
      ),
      onViewCreated: (viewId) async {
        String userId = index.toString();
        if (remoteUidList.length > index) {
          userId = remoteUidList[index];
        }

        if (remoteUserViewMap.containsKey(userId)) {
          baiduRtcCloud.updateDisplay(
              viewId, int.parse(userId));
        } else {
          baiduRtcCloud.setRemoteDisplayWithUserId(
              viewId, int.parse(userId));
          remoteUserViewMap[userId] = viewId;
        }
      },
    );
    if (remoteUidList.length > index) {
      remoteUserIdVideoViewMap[remoteUidList[index]] = remoteView;
    }
    return remoteView;
  }

  @override
  Widget build(BuildContext context) {
    final double distance = 12;
    List<String> remoteUidList = remoteUserIdMap.values.toList();
    subscriberCount = remoteUidList.length;
    return Stack(
      alignment: Alignment.topLeft,
      fit: StackFit.expand,
      children: [
        Positioned(
          key: ValueKey("Background"),
          top: 0,
          left: 0,
          child: Container(
            width: 1.sw,
            height: 1.sh,
            color: Colors.black,
          ),
        ),
        Positioned(
          top: 0,
          left: 0,
          child: Container(
            width: 1.sw,
            height: (4 / 3).sw,
            color: Colors.black,
            child: BRTCCloudVideoView(
              key: ValueKey("LocalView"),
              viewType: BRTCCloudConfig.BRTC_Video_SurfaceView_Channel,
              videoParam: BRTCVideoParams(
                x: 0,
                y: 0,
                width: 1.sw.toInt(),
                height: (4 / 3).sw.toInt(),
                isLocal: true,
              ),
              onViewCreated: (viewId) async {
                setState(() {
                  localViewId = viewId;
                });
                baiduRtcCloud.setLocalDisplay(viewId);
              },
            ),
          ),
        ),
        Row(
          mainAxisAlignment: MainAxisAlignment.end,
          mainAxisSize: MainAxisSize.min,
          children: [
            Container(
              alignment: Alignment.topRight,
              color: Colors.transparent,
              width: (1 / 3).sw,
              child: GridView.builder(
                itemCount: subscriberCount,
                shrinkWrap: false,
                gridDelegate: SliverGridDelegateWithFixedCrossAxisCount(
                    // 横轴元素个数
                    crossAxisCount: 1,
                    // 纵轴间距
                    mainAxisSpacing: 2.0,
                    // 横轴间距
                    crossAxisSpacing: 5.0,
                    // 子组件宽高长度比例
                    childAspectRatio: 0.75),
                itemBuilder: (BuildContext context, int index) {
                  return Container(
                    child: getRemoteView(remoteUidList, index),
                  );
                },
              ),
            ),
          ],
        ),
        Positioned(
          left: 6,
          right: 6,
          bottom: 10,
          child: Row(
            mainAxisAlignment: MainAxisAlignment.spaceEvenly,
            children: [
              Row(
                children: [
                  PhotoTextView(
                    key: ValueKey("muteAudio" + isMuteLocalAudio.toString()),
                    text: "静音",
                    imagePath: isMuteLocalAudio
                        ? "assets/images/callfunc/btn_mute_selected.png"
                        : "assets/images/callfunc/btn_mute.png",
                    clickListener: () {
                      bool newIsMuteLocalAudio = !isMuteLocalAudio;
                      baiduRtcCloud.muteMicphone(newIsMuteLocalAudio);
                      setState(() {
                        isMuteLocalAudio = newIsMuteLocalAudio;
                      });
                    },
                  ),
                  SizedBox(
                    width: distance,
                  ),
                  PhotoTextView(
                    key: ValueKey("speaker" + isSpeaker.toString()),
                    text: "扬声器",
                    imagePath: isSpeaker
                        ? "assets/images/callfunc/btn_speaker_selected.png"
                        : "assets/images/callfunc/btn_speaker.png",
                    clickListener: () {
                      bool newIsSpeaker = !isSpeaker;
                      baiduRtcCloud.switchLoundSpeaker();
                      setState(() {
                        isSpeaker = newIsSpeaker;
                      });
                    },
                  ),
                  SizedBox(
                    width: distance,
                  ),
                  PhotoTextView(
                    key: ValueKey("cancel"),
                    text: "挂断",
                    imagePath: "assets/images/callfunc/btn_end_call.png",
                    clickListener: () {
                      // 挂断
                      Navigator.pop(context);
                    },
                  ),
                  SizedBox(
                    width: distance,
                  ),
                  PhotoTextView(
                    key: ValueKey("openCamera" + isOpenCamera.toString()),
                    text: "切到语音",
                    imagePath: !isOpenCamera
                        ? "assets/images/callfunc/btn_voice_selected.png"
                        : "assets/images/callfunc/btn_voice.png",
                    clickListener: () {
                      bool newIsOpenCamera = isOpenCamera;
                      baiduRtcCloud.muteCamera(newIsOpenCamera);
                      setState(() {
                        isOpenCamera = !newIsOpenCamera;
                      });
                    },
                  ),
                  SizedBox(
                    width: distance,
                  ),
                  PhotoTextView(
                    key: ValueKey("frontCamera" + isFrontCamera.toString()),
                    text: "摄像头",
                    imagePath: !isFrontCamera
                        ? "assets/images/callfunc/btn_switch_camera_selected.png"
                        : "assets/images/callfunc/btn_switch_camera.png",
                    clickListener: () {
                      bool newIsFrontCamera = !isFrontCamera;
                      baiduRtcCloud.switchCamera();
                      setState(() {
                        isFrontCamera = newIsFrontCamera;
                      });
                    },
                  ),
                ],
              ),
            ],
          ),
        ),
      ],
    );
  }
}
