import 'package:flutter/material.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';

import 'IEventCallback.dart';

class FunctionPage extends StatefulWidget {
  final IEventCallback iEventCallback;

  FunctionPage(this.iEventCallback);

  @override
  _FunctionPageState createState() => _FunctionPageState(this.iEventCallback);
}

class _FunctionPageState extends State<FunctionPage>
    with TickerProviderStateMixin, AutomaticKeepAliveClientMixin {
  final IEventCallback iEventCallback;

  _FunctionPageState(this.iEventCallback);

  List tabList = [
    "视频",
    "跨房连麦",
    "流操作",
    "控制",
    "发消息",
  ];
  late TabController _tabController;
  late TextEditingController _editController;

  final focusNodeUserId = FocusNode();

  @protected
  bool get wantKeepAlive => true;

  @override
  void initState() {
    super.initState();
    _tabController =
        TabController(initialIndex: 0, length: tabList.length, vsync: this);
    _editController = new TextEditingController();

    if (_tabController.indexIsChanging) {
      print("---->indexch");
    }
  }

  @override
  void dispose() {
    cancelFocus();
    super.dispose();
  }

  void cancelFocus() {
    if (focusNodeUserId.hasFocus) {
      focusNodeUserId.unfocus();
    }
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      resizeToAvoidBottomInset: false,
      backgroundColor: Colors.white,
      body: GestureDetector(
        onTap: () {
          cancelFocus();
        },
        child: Column(
          children: <Widget>[
            Container(
              alignment: Alignment.center,
              child: TabBar(
                indicatorColor: Colors.blueAccent,
                controller: _tabController,
                isScrollable: true,
                tabs: tabList.map((text) {
                  return Container(
                    padding: EdgeInsets.only(
                        top: 10.sp, bottom: 10.sp, left: 0, right: 0),
                    alignment: Alignment(0, 0),
                    child: Text(
                      text,
                      style: TextStyle(color: Colors.black, fontSize: 26.sp),
                    ),
                  );
                }).toList(),
              ),
            ),
            Expanded(
              child: TabBarView(
                  controller: _tabController,
                  children: tabList.isEmpty
                      ? []
                      : tabList.map((text) {
                          Widget widget = Center(
                            child: new Text("开发中",
                                style: TextStyle(
                                    color: Colors.black, fontSize: 30.sp)),
                          );
                          if (text == tabList[0]) {
                            return widget;
                          } else if (text == tabList[1]) {
                            return widget;
                          } else if (text == tabList[2]) {
                            return getStreamControlPage();
                          } else if (text == tabList[3]) {
                            return widget;
                          } else if (text == tabList[4]) {
                            return widget;
                          } else {
                            return widget;
                          }
                        }).toList()),
            ),
          ],
        ),
      ),
    );
  }

  Widget getStreamControlPage() {
    return Padding(
      padding:
          EdgeInsets.only(top: 30.sp, bottom: 0, left: 20.sp, right: 20.sp),
      child: Column(
        crossAxisAlignment: CrossAxisAlignment.start,
        children: [
          Text(
            "手动订阅",
            style: TextStyle(
              color: Colors.black,
              fontSize: 30.sp,
            ),
          ),
          SizedBox(height: 35.sp),
          Container(
            alignment: Alignment(0, 0),
            height: 60.sp,
            width: 1.sw,
            child: Row(
              mainAxisAlignment: MainAxisAlignment.spaceBetween,
              children: <Widget>[
                Expanded(
                  child: Container(
                    padding:
                        EdgeInsets.only(top: 0, bottom: 0, left: 0, right: 0),
                    alignment: Alignment.centerLeft,
                    child: TextField(
                      controller: _editController,
                      autofocus: false,
                      focusNode: focusNodeUserId,
                      decoration: InputDecoration(
                        hintText: '输入userId',
                        hintStyle: TextStyle(
                            color: Color(0xFFB8BABF), fontSize: 30.sp),
                        hintMaxLines: 1,
                        enabledBorder: new UnderlineInputBorder(
                          borderSide: BorderSide(color: Color(0x19000000)),
                        ),
                        focusedBorder: new UnderlineInputBorder(
                          borderSide: BorderSide(color: Colors.blueAccent),
                        ),
                      ),
                      style: TextStyle(
                        color: Colors.black,
                        fontSize: 35.sp,
                      ),
                    ),
                  ),
                ),
                GestureDetector(
                  child: Container(
                    padding: EdgeInsets.only(
                        top: 0, bottom: 0, left: 15.sp, right: 15.sp),
                    alignment: Alignment.centerRight,
                    height: 60.sp,
                    decoration: BoxDecoration(
                      border: new Border.all(color: Colors.blue, width: 1),
                      borderRadius: BorderRadius.only(
                        bottomLeft: const Radius.circular(3.0),
                        bottomRight: const Radius.circular(3.0),
                        topLeft: const Radius.circular(3.0),
                        topRight: const Radius.circular(3.0),
                      ),
                    ),
                    child: Text(
                      "订阅",
                      style: TextStyle(color: Colors.blue, fontSize: 26.sp),
                    ),
                  ),
                  onTap: () {
                    if (iEventCallback != null &&
                        _editController.text.isNotEmpty) {
                      iEventCallback.onSubscribe(_editController.text);
                    }
                  },
                ),
                SizedBox(width: 10.sp),
                GestureDetector(
                  child: Container(
                    padding: EdgeInsets.only(
                        top: 0, bottom: 0, left: 15.sp, right: 15.sp),
                    alignment: Alignment.centerRight,
                    height: 60.sp,
                    decoration: BoxDecoration(
                      border: new Border.all(color: Colors.blue, width: 1),
                      borderRadius: BorderRadius.only(
                        bottomLeft: const Radius.circular(3.0),
                        bottomRight: const Radius.circular(3.0),
                        topLeft: const Radius.circular(3.0),
                        topRight: const Radius.circular(3.0),
                      ),
                    ),
                    child: Text(
                      "取消订阅",
                      style: TextStyle(color: Colors.blue, fontSize: 26.sp),
                    ),
                  ),
                  onTap: () {
                    if (iEventCallback != null &&
                        _editController.text.isNotEmpty) {
                      iEventCallback.onStopSubscribe(_editController.text);
                    }
                  },
                ),
              ],
            ),
          ),
          Container(
            height: 15.sp,
            color: Color(0xFFF7F7F9),
            margin:
                EdgeInsets.only(top: 40.sp, bottom: 40.sp, left: 0, right: 0),
          ),
          Container(
            alignment: Alignment(0, 0),
            height: 100.sp,
            width: 1.sw,
            child: Row(
              mainAxisAlignment: MainAxisAlignment.spaceBetween,
              children: <Widget>[
                Expanded(
                  child: Container(
                    padding:
                        EdgeInsets.only(top: 0, bottom: 0, left: 0, right: 0),
                    alignment: Alignment.centerLeft,
                    child: Text(
                      "手动发布",
                      style: TextStyle(
                        color: Colors.black,
                        fontSize: 30.sp,
                      ),
                    ),
                  ),
                ),
                GestureDetector(
                  child: Container(
                    padding: EdgeInsets.only(
                        top: 0, bottom: 0, left: 15.sp, right: 15.sp),
                    alignment: Alignment.centerRight,
                    height: 60.sp,
                    decoration: BoxDecoration(
                      border: new Border.all(color: Colors.blue, width: 1),
                      borderRadius: BorderRadius.only(
                        bottomLeft: const Radius.circular(3.0),
                        bottomRight: const Radius.circular(3.0),
                        topLeft: const Radius.circular(3.0),
                        topRight: const Radius.circular(3.0),
                      ),
                    ),
                    child: Text(
                      "    发布    ",
                      style: TextStyle(color: Colors.blue, fontSize: 26.sp),
                    ),
                  ),
                  onTap: () {
                    if (iEventCallback != null) {
                      iEventCallback.onPublish();
                    }
                  },
                ),
              ],
            ),
          ),
        ],
      ),
    );
  }
}
