import 'package:flutter/material.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';
import 'package:oktoast/oktoast.dart';

import 'FunctionPage.dart';
import 'IEventCallback.dart';

class TopFunctionView extends StatefulWidget {
  final double width;
  final double height;
  final IEventCallback? iEventCallback;

  const TopFunctionView(
      {Key? key,
      required this.width,
      required this.height,
      this.iEventCallback})
      : super(key: key);

  @override
  State<StatefulWidget> createState() => TopFunctionViewState(
      super.key, this.width, this.height, this.iEventCallback!);
}

class TopFunctionViewState extends State<StatefulWidget> {
  final double width;
  final double height;
  final IEventCallback iEventCallback;

  TopFunctionViewState(Key? key, this.width, this.height, this.iEventCallback);

  @override
  void setState(VoidCallback fn) {
    super.setState(fn);
  }

  @override
  Widget build(BuildContext context) {
    final double distance = 15.sp;
    final double size = height;
    return Container(
      alignment: Alignment(0, 0),
      child: Row(
        mainAxisAlignment: MainAxisAlignment.spaceBetween,
        children: <Widget>[
          Expanded(
            child: InkWell(
              child: Container(
                padding:
                    EdgeInsets.only(top: 0, bottom: 0, left: 20.sp, right: 0),
                alignment: Alignment.centerLeft,
                width: size,
                height: size,
                child: Image.asset("assets/images/roomfunc/icon_room_log.png"),
              ),
              onTap: () {
                showToast('开发中');
              },
            ),
          ),
          Container(
            padding: EdgeInsets.only(top: 0, bottom: 0, left: 0, right: 20.sp),
            alignment: Alignment.centerRight,
            child: Row(
              mainAxisAlignment: MainAxisAlignment.spaceEvenly,
              children: [
                Row(
                  children: [
                    InkWell(
                      child: Container(
                        alignment: Alignment.center,
                        width: size,
                        height: size,
                        child: Image.asset(
                            "assets/images/roomfunc/icon_room_sharescreen.png"),
                      ),
                      onTap: () {
                        showToast('开发中');
                      },
                    ),
                    SizedBox(
                      width: distance,
                    ),
                    InkWell(
                      child: Container(
                        alignment: Alignment.center,
                        width: size,
                        height: size,
                        child: Image.asset(
                            "assets/images/roomfunc/icon_room_userlist.png"),
                      ),
                      onTap: () {
                        showToast('开发中');
                      },
                    ),
                    SizedBox(
                      width: distance,
                    ),
                    InkWell(
                      child: Container(
                        alignment: Alignment.center,
                        width: size,
                        height: size,
                        child: Image.asset(
                            "assets/images/roomfunc/icon_room_beauty.png"),
                      ),
                      onTap: () {
                        showToast('开发中');
                      },
                    ),
                    SizedBox(
                      width: distance,
                    ),
                    InkWell(
                      child: Container(
                        alignment: Alignment.center,
                        width: size,
                        height: size,
                        child: Image.asset(
                            "assets/images/roomfunc/icon_room_setting.png"),
                      ),
                      onTap: () {
                        _showCustomModalBottomSheet(context);
                      },
                    ),
                  ],
                ),
              ],
            ),
          ),
        ],
      ),
    );
  }

  Future<int?> _showCustomModalBottomSheet(context) async {
    return showModalBottomSheet<int>(
      backgroundColor: Colors.transparent,
      isScrollControlled: true,
      context: context,
      builder: (BuildContext context) {
        return Container(
          clipBehavior: Clip.antiAlias,
          decoration: BoxDecoration(
            color: Colors.white,
            borderRadius: BorderRadius.only(
              topLeft: const Radius.circular(10.0),
              topRight: const Radius.circular(10.0),
            ),
          ),
          height: MediaQuery.of(context).size.height / 1.5,
          child: Column(children: [
            Stack(
              textDirection: TextDirection.rtl,
              children: [
                Container(
                  padding: EdgeInsets.only(
                      top: 30.sp, bottom: 30.sp, left: 0, right: 0),
                  alignment: Alignment(0, 0),
                  child: Text(
                    '设置',
                    style: TextStyle(
                        color: Colors.black,
                        fontWeight: FontWeight.bold,
                        fontSize: 35.0.sp),
                  ),
                ),
                IconButton(
                    icon: Icon(Icons.close),
                    onPressed: () {
                      Navigator.of(context).pop();
                    }),
              ],
            ),
            Divider(height: 1.0),
            Expanded(
              child: FunctionPage(this.iEventCallback),
            ),
          ]),
        );
      },
    );
  }
}
