import 'package:brtc_flutter_demo/BdUtils.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';

import '../route/ExampleData.dart';

//  BRTC AudioSettingPage.dart
class AudioSettingPage extends StatefulWidget {
  AudioSettingPage({Key? key}) : super(key: key);

  @override
  _AudioSettingPageState createState() => _AudioSettingPageState();
}

class _AudioSettingPageState extends State<AudioSettingPage> {
  final List<ExampleItem> settingList = [
    ExampleItem(
        title: "人声增益", color: Colors.white, on: BdUtils.getAudioBdAGCConfig()),
    ExampleItem(
        title: "回声消除", color: Colors.white, on: BdUtils.getAudioBdAECConfig()),
    ExampleItem(
        title: "噪声抑制", color: Colors.white, on: BdUtils.getAudioBdNSConfig()),
  ];

  Widget getSeparatorBuilder(BuildContext context, index) {
    Divider divider = Divider(
      thickness: 0.5.sp,
      color: Colors.grey,
    );
    return divider;
  }

  Widget getItemBuilder(context, index) {
    ExampleItem currentItem = this.settingList[index];

    GestureDetector itemWidget = GestureDetector(
      child: Container(
        alignment: Alignment(0, 0),
        height: 100.sp,
        width: 1.sw,
        child: Row(
          mainAxisAlignment: MainAxisAlignment.spaceBetween,
          children: <Widget>[
            Expanded(
              child: Container(
                padding:
                    EdgeInsets.only(top: 0, bottom: 0, left: 25.sp, right: 0),
                alignment: Alignment.centerLeft,
                color: currentItem.color,
                child: Text(
                  currentItem.title,
                  style: TextStyle(color: Colors.black, fontSize: 35.sp),
                ),
              ),
            ),
            Container(
              padding:
                  EdgeInsets.only(top: 0, bottom: 0, left: 0, right: 20.sp),
              alignment: Alignment.centerRight,
              child: Switch(
                  value: currentItem.on!,
                  activeColor: Colors.blueAccent,
                  activeTrackColor: Colors.grey,
                  inactiveThumbColor: Colors.white,
                  inactiveTrackColor: Colors.black12,
                  onChanged: (value) {
                    setState(() {
                      currentItem.on = value;
                      if (index == 0) {
                        BdUtils.setAudioBdAGCConfig(value);
                      } else if (index == 1) {
                        BdUtils.setAudioBdAECConfig(value);
                      } else {
                        BdUtils.setAudioBdNSConfig(value);
                      }
                    });
                  }),
            ),
          ],
        ),
      ),
    );
    return itemWidget;
  }

  @override
  void initState() {
    super.initState();
  }

  @override
  void dispose() {
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      width: 1.sw,
      height: 1.sh,
      color: Colors.white,
      margin: EdgeInsets.only(top: 10),
      padding: EdgeInsets.only(top: 0, bottom: 0, left: 0, right: 0),
      child: ListView.separated(
        separatorBuilder: this.getSeparatorBuilder,
        itemBuilder: this.getItemBuilder,
        itemCount: settingList.length,
      ),
    );
  }
}
