import 'package:brtc_flutter_demo/BdUtils.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';

import '../route/ExampleData.dart';

//  BRTC OtherSettingPage.dart
class OtherSettingPage extends StatefulWidget {
  OtherSettingPage({Key? key}) : super(key: key);

  @override
  _OtherSettingPageState createState() => _OtherSettingPageState();
}

class _OtherSettingPageState extends State<OtherSettingPage> {
  String signalServerUrl = BdUtils.getSignalServerUrl();
  String appId = BdUtils.getAppId();
  String token = BdUtils.getToken();
  bool autoPublish = BdUtils.getAutoPublishConfig();
  bool autoSubscribe = BdUtils.getAutoSubscribeConfig();

  final focusNodeUrl = FocusNode();
  final focusNodeAppId = FocusNode();
  final focusNodeToken = FocusNode();

  final List<ExampleItem> settingList = [
    ExampleItem(
        title: "自动发布", color: Colors.white, on: BdUtils.getAutoPublishConfig()),
    ExampleItem(
        title: "自动订阅",
        color: Colors.white,
        on: BdUtils.getAutoSubscribeConfig()),
  ];

  Widget getSeparatorBuilder(BuildContext context, index) {
    Divider divider = Divider(
      thickness: 0.5.sp,
      color: Colors.grey,
    );
    return divider;
  }

  Widget getSizeBox({height, Widget? child}) {
    return SizedBox(height: height != null ? height : 25.sp, child: child);
  }

  @override
  void initState() {
    super.initState();
  }

  void cancelFocus() {
    if (focusNodeUrl.hasFocus) {
      focusNodeUrl.unfocus();
    } else if (focusNodeAppId.hasFocus) {
      focusNodeAppId.unfocus();
    } else if (focusNodeToken.hasFocus) {
      focusNodeToken.unfocus();
    }
  }

  @override
  void dispose() {
    cancelFocus();
    super.dispose();
    BdUtils.setSignalServerUrl(signalServerUrl);
    BdUtils.setAppId(appId);
    BdUtils.setToken(token);
    BdUtils.setAutoPublishConfig(autoPublish);
    BdUtils.setAutoSubscribeConfig(autoSubscribe);
  }

  @override
  Widget build(BuildContext context) {
    return GestureDetector(
      behavior: HitTestBehavior.translucent,
      onTap: () {
        cancelFocus();
      },
      child: Container(
        width: 1.sw,
        height: 1.sh,
        color: Colors.white,
        margin: EdgeInsets.only(top: 10),
        padding: EdgeInsets.only(top: 0, bottom: 0, left: 20, right: 20),
        child: Column(
          children: [
            TextField(
              style: TextStyle(color: Colors.blueAccent),
              autofocus: false,
              controller: TextEditingController.fromValue(
                TextEditingValue(
                  text: this.signalServerUrl,
                  selection: TextSelection.fromPosition(
                    TextPosition(
                        affinity: TextAffinity.downstream,
                        offset: '${this.token}'.length),
                  ),
                ),
              ),
              focusNode: focusNodeUrl,
              decoration: InputDecoration(
                labelText: "信令服务器地址",
                hintText: "请输入地址",
                labelStyle: TextStyle(color: Colors.black),
                hintStyle: TextStyle(color: Color.fromRGBO(44, 118, 255, 1)),
                enabledBorder: UnderlineInputBorder(
                  borderSide: BorderSide(color: Colors.black, width: 0.5),
                ),
              ),
              keyboardType: TextInputType.text,
              onChanged: (value) {
                this.signalServerUrl = value;
              },
            ),
            getSizeBox(),
            TextField(
              style: TextStyle(color: Colors.blueAccent),
              autofocus: false,
              controller: TextEditingController.fromValue(
                TextEditingValue(
                  text: this.appId.toString(),
                  selection: TextSelection.fromPosition(
                    TextPosition(
                        affinity: TextAffinity.downstream,
                        offset: '${this.appId}'.length),
                  ),
                ),
              ),
              focusNode: focusNodeAppId,
              decoration: InputDecoration(
                labelText: "AppID",
                hintText: "请输入AppID",
                labelStyle: TextStyle(color: Colors.black),
                hintStyle: TextStyle(color: Color.fromRGBO(44, 118, 255, 1)),
                enabledBorder: UnderlineInputBorder(
                  borderSide: BorderSide(color: Colors.black, width: 0.5),
                ),
              ),
              keyboardType: TextInputType.number,
              onChanged: (value) {
                appId = value;
              },
            ),
            getSizeBox(),
            TextField(
              style: TextStyle(color: Colors.blueAccent),
              autofocus: false,
              controller: TextEditingController.fromValue(
                TextEditingValue(
                  text: this.token,
                  selection: TextSelection.fromPosition(
                    TextPosition(
                        affinity: TextAffinity.downstream,
                        offset: '${this.token}'.length),
                  ),
                ),
              ),
              focusNode: focusNodeToken,
              decoration: InputDecoration(
                labelText: "Token",
                hintText: "请输入Token",
                labelStyle: TextStyle(color: Colors.black),
                hintStyle: TextStyle(color: Color.fromRGBO(44, 118, 255, 1)),
                enabledBorder: UnderlineInputBorder(
                  borderSide: BorderSide(color: Colors.black, width: 0.5),
                ),
              ),
              keyboardType: TextInputType.text,
              onChanged: (value) {
                this.token = value;
              },
            ),
            getSizeBox(height: 100.sp),
            GestureDetector(
              child: Container(
                alignment: Alignment(0, 0),
                height: 100.sp,
                width: 1.sw,
                child: Row(
                  mainAxisAlignment: MainAxisAlignment.spaceBetween,
                  children: <Widget>[
                    Expanded(
                      child: Container(
                        alignment: Alignment.centerLeft,
                        color: Colors.white,
                        child: Text(
                          "自动发布",
                          style:
                              TextStyle(color: Colors.black, fontSize: 35.sp),
                        ),
                      ),
                    ),
                    Container(
                      padding:
                          EdgeInsets.only(top: 0, bottom: 0, left: 0, right: 0),
                      alignment: Alignment.centerRight,
                      child: Switch(
                          value: autoPublish,
                          activeColor: Colors.blueAccent,
                          activeTrackColor: Colors.grey,
                          inactiveThumbColor: Colors.white,
                          inactiveTrackColor: Colors.black12,
                          onChanged: (value) {
                            setState(() {
                              autoPublish = value;
                            });
                          }),
                    ),
                  ],
                ),
              ),
            ),
            getSizeBox(height: 0.8.sp, child: Container(color: Colors.black)),
            getSizeBox(height: 30.sp),
            GestureDetector(
              child: Container(
                alignment: Alignment(0, 0),
                height: 100.sp,
                width: 1.sw,
                child: Row(
                  mainAxisAlignment: MainAxisAlignment.spaceBetween,
                  children: <Widget>[
                    Expanded(
                      child: Container(
                        alignment: Alignment.centerLeft,
                        color: Colors.white,
                        child: Text(
                          "自动订阅",
                          style:
                              TextStyle(color: Colors.black, fontSize: 35.sp),
                        ),
                      ),
                    ),
                    Container(
                      alignment: Alignment.centerRight,
                      child: Switch(
                          value: autoSubscribe,
                          activeColor: Colors.blueAccent,
                          activeTrackColor: Colors.grey,
                          inactiveThumbColor: Colors.white,
                          inactiveTrackColor: Colors.black12,
                          onChanged: (value) {
                            setState(() {
                              autoSubscribe = value;
                            });
                          }),
                    ),
                  ],
                ),
              ),
            ),
          ],
        ),
      ),
    );
  }
}
