import 'package:brtc_flutter_demo/settings/AudioSettingPage.dart';
import 'package:brtc_flutter_demo/settings/OtherSettingPage.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';
import 'package:oktoast/oktoast.dart';

import '../route/ExampleData.dart';
import '../route/ExampleDataLayout.dart';
import 'VideoSettingPage.dart';

//  BRTC SettingsPage.dart
class SettingsPage extends StatefulWidget {
  SettingsPage({Key? key}) : super(key: key);

  @override
  _SettingsPageState createState() => _SettingsPageState();
}

class _SettingsPageState extends State<SettingsPage> {
  final List<ExampleItem> settingList = [
    ExampleItem(
        title: "媒体参数设置",
        color: Colors.white,
        detailPage: VideoSettingPage(),
        logoPath: "assets/images/settings/arrow_right.png"),
    ExampleItem(
        title: "音频处理",
        color: Colors.white,
        detailPage: AudioSettingPage(),
        logoPath: "assets/images/settings/arrow_right.png"),
    ExampleItem(
        title: "混流设置",
        color: Colors.white,
        logoPath: "assets/images/settings/arrow_right.png"),
    ExampleItem(
        title: "其他",
        color: Colors.white,
        detailPage: OtherSettingPage(),
        logoPath: "assets/images/settings/arrow_right.png"),
  ];

  Widget getSeparatorBuilder(BuildContext context, index) {
    Divider divider = Divider(
      thickness: 0.5.sp,
      color: Colors.grey,
    );
    return divider;
  }

  Widget getItemBuilder(context, index) {
    ExampleItem currentItem = this.settingList[index];
    GestureDetector itemWidget = GestureDetector(
      child: Container(
        alignment: Alignment(0, 0),
        height: 100.sp,
        width: 1.sw,
        child: Row(
          mainAxisAlignment: MainAxisAlignment.spaceBetween,
          children: <Widget>[
            Expanded(
              child: Container(
                padding:
                    EdgeInsets.only(top: 0, bottom: 0, left: 20.sp, right: 0),
                alignment: Alignment.centerLeft,
                color: currentItem.color,
                child: Text(
                  currentItem.title,
                  style: TextStyle(color: Colors.black, fontSize: 35.sp),
                ),
              ),
            ),
            Container(
              padding:
                  EdgeInsets.only(top: 0, bottom: 0, left: 0, right: 20.sp),
              alignment: Alignment.centerRight,
              width: 50.sp,
              height: 50.sp,
              child: Image.asset(currentItem.logoPath!),
            ),
          ],
        ),
      ),
      onTap: () {
        if (currentItem.detailPage != null) {
          Navigator.push(
            context,
            new MaterialPageRoute(
              builder: (context) => ExampleDataLayout(
                examplePageData: currentItem,
              ),
            ),
          );
        } else {
          showToast('开发中');
        }
      },
    );
    return itemWidget;
  }

  @override
  void initState() {
    super.initState();
  }

  @override
  void dispose() {
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      width: 1.sw,
      height: 1.sh,
      color: Colors.white,
      margin: EdgeInsets.only(top: 10),
      padding: EdgeInsets.only(top: 0, bottom: 0, left: 0, right: 0),
      child: ListView.separated(
        separatorBuilder: this.getSeparatorBuilder,
        itemBuilder: this.getItemBuilder,
        itemCount: settingList.length,
      ),
    );
  }
}
