import 'package:brtc_flutter_demo/BdUtils.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';

import '../route/ExampleData.dart';

//  BRTC VideoSettingPage.dart
class VideoSettingPage extends StatefulWidget {
  VideoSettingPage({Key? key}) : super(key: key);

  @override
  _VideoSettingPageState createState() => _VideoSettingPageState();
}

class _VideoSettingPageState extends State<VideoSettingPage> {
  final focusNodeBitrate = FocusNode();
  final focusNodeFps = FocusNode();

  int videoBitrate = BdUtils.getBitrateConfig();
  int videoFps = BdUtils.getFpsConfig();
  String codecType = BdUtils.getCodecTypeConfig();
  String resolution = BdUtils.getResolutionConfig();

  final List<ExampleItem> codecList = [
    ExampleItem(
        title: "H264",
        color: Colors.white,
        on: "H264" == BdUtils.getCodecTypeConfig() ? true : false,
        currentValue: BdUtils.getCodecTypeConfig(),
        logoPath: "assets/images/settings/icon_select.png"),
    // ExampleItem(
    //     title: "H263",
    //     color: Colors.white,
    //     on: "H263" == BdUtils.getCodecTypeConfig() ? true : false,
    //     currentValue: BdUtils.getCodecTypeConfig(),
    //     logoPath: "assets/images/settings/icon_select.png"),
  ];

  final List<ExampleItem> resolutionList = [
    ExampleItem(
        title: "1920x1080",
        color: Colors.white,
        on: "1920x1080" == BdUtils.getResolutionConfig() ? true : false,
        currentValue: BdUtils.getResolutionConfig(),
        logoPath: "assets/images/settings/icon_select.png"),
    ExampleItem(
        title: "1280x720",
        color: Colors.white,
        on: "1280x720" == BdUtils.getResolutionConfig() ? true : false,
        currentValue: BdUtils.getResolutionConfig(),
        logoPath: "assets/images/settings/icon_select.png"),
    ExampleItem(
        title: "640x480",
        color: Colors.white,
        on: "640x480" == BdUtils.getResolutionConfig() ? true : false,
        currentValue: BdUtils.getResolutionConfig(),
        logoPath: "assets/images/settings/icon_select.png"),
    ExampleItem(
        title: "640x360",
        color: Colors.white,
        on: "640x360" == BdUtils.getResolutionConfig() ? true : false,
        currentValue: BdUtils.getResolutionConfig(),
        logoPath: "assets/images/settings/icon_select.png"),
    ExampleItem(
        title: "352x288",
        color: Colors.white,
        on: "352x288" == BdUtils.getResolutionConfig() ? true : false,
        currentValue: BdUtils.getResolutionConfig(),
        logoPath: "assets/images/settings/icon_select.png"),
  ];

  Widget getTextWidget(text, {margin}) {
    return Container(
      margin: margin,
      width: 1.sw,
      color: Color(0xFFF7F7F9),
      padding: EdgeInsets.only(left: 20, top: 18.sp, right: 0, bottom: 18.sp),
      child: Text(
        text,
        style: TextStyle(color: Color(0xFF5C5F66), fontSize: 26.sp),
      ),
    );
  }

  Widget getSeparatorBuilder(BuildContext context, index) {
    Divider divider = Divider(
      thickness: 0.5.sp,
      color: Colors.grey,
    );
    return divider;
  }

  Widget getCurrentItem(context, index, dataList) {
    ExampleItem currentItem = dataList[index];
    GestureDetector itemWidget = GestureDetector(
      child: Container(
        alignment: Alignment(0, 0),
        height: 90.sp,
        width: 1.sw,
        child: Row(
          mainAxisAlignment: MainAxisAlignment.spaceBetween,
          children: <Widget>[
            Expanded(
              child: Container(
                padding: EdgeInsets.only(top: 0, bottom: 0, left: 0, right: 0),
                alignment: Alignment.centerLeft,
                color: currentItem.color,
                child: Text(
                  currentItem.title,
                  style: TextStyle(color: Colors.black, fontSize: 32.sp),
                ),
              ),
            ),
            Container(
              padding: EdgeInsets.only(top: 0, bottom: 0, left: 0, right: 0),
              alignment: Alignment.centerRight,
              width: 50.sp,
              height: 50.sp,
              child: (currentItem.on!)
                  ? Image.asset(currentItem.logoPath!)
                  : Container(),
            ),
          ],
        ),
      ),
      onTap: () {
        setState(() {
          for (int i = 0; i < dataList.length; i++) {
            if (i == index) {
              dataList[i].on = true;
            } else {
              dataList[i].on = false;
            }
            dataList[i].currentValue = dataList[index].title;
          }
        });
      },
    );
    return itemWidget;
  }

  Widget getCodecItem(context, index) {
    return getCurrentItem(context, index, codecList);
  }

  Widget getResolutionItem(context, index) {
    return getCurrentItem(context, index, resolutionList);
  }

  @override
  void initState() {
    super.initState();
  }

  @override
  void dispose() {
    cancelFocus();
    super.dispose();
    BdUtils.setCodecTypeConfig(codecList[0].currentValue!);
    BdUtils.setResolutionConfig(resolutionList[0].currentValue!);
    BdUtils.setBitrateConfig(videoBitrate);
    BdUtils.setFpsConfig(videoFps);
  }

  void cancelFocus() {
    if (focusNodeBitrate.hasFocus) {
      focusNodeBitrate.unfocus();
    } else if (focusNodeFps.hasFocus) {
      focusNodeFps.unfocus();
    }
  }

  @override
  Widget build(BuildContext context) {
    return GestureDetector(
        behavior: HitTestBehavior.translucent,
        onTap: () {
          cancelFocus();
        },
        child: Container(
          width: 1.sw,
          height: 1.sh,
          color: Colors.white,
          child: Column(
            crossAxisAlignment: CrossAxisAlignment.start,
            children: [
              getTextWidget("编解码"),
              Container(
                padding:
                    EdgeInsets.only(top: 0, bottom: 0, left: 20, right: 20),
                width: 1.sw,
                height: 110.sp,
                color: Colors.white,
                child: ListView.separated(
                  separatorBuilder: this.getSeparatorBuilder,
                  itemBuilder: this.getCodecItem,
                  itemCount: codecList.length,
                ),
              ),
              getTextWidget("最大码率（Kbps）"),
              Container(
                width: 1.sw,
                padding:
                    EdgeInsets.only(top: 0, bottom: 0, left: 20, right: 20),
                child: TextField(
                  controller: TextEditingController.fromValue(
                    TextEditingValue(
                      text: this.videoBitrate.toString(),
                      selection: TextSelection.fromPosition(
                        TextPosition(
                            affinity: TextAffinity.downstream,
                            offset: '${this.videoBitrate}'.length),
                      ),
                    ),
                  ),
                  autofocus: false,
                  focusNode: focusNodeBitrate,
                  decoration: InputDecoration(
                    hintText: '最大码率',
                    hintStyle:
                        TextStyle(color: Color(0xFFB8BABF), fontSize: 30.sp),
                    hintMaxLines: 1,
                    enabledBorder: new UnderlineInputBorder(
                      borderSide: BorderSide(color: Color(0x19000000)),
                    ),
                    focusedBorder: new UnderlineInputBorder(
                      borderSide: BorderSide(color: Colors.blueAccent),
                    ),
                  ),
                  style: TextStyle(
                    color: Colors.black,
                    fontSize: 32.sp,
                  ),
                  keyboardType: TextInputType.text,
                  onChanged: (value) {
                    this.videoBitrate = int.parse(value);
                  },
                ),
              ),
              getTextWidget("帧率（FPS)",
                  margin: EdgeInsets.only(
                      left: 0, top: 15.sp, right: 0, bottom: 0)),
              Container(
                width: 1.sw,
                padding:
                    EdgeInsets.only(top: 0, bottom: 0, left: 20, right: 20),
                child: TextField(
                  controller: TextEditingController.fromValue(
                    TextEditingValue(
                      text: this.videoFps.toString(),
                      selection: TextSelection.fromPosition(
                        TextPosition(
                            affinity: TextAffinity.downstream,
                            offset: '${this.videoFps}'.length),
                      ),
                    ),
                  ),
                  autofocus: false,
                  focusNode: focusNodeFps,
                  decoration: InputDecoration(
                    hintText: '帧率',
                    hintStyle:
                        TextStyle(color: Color(0xFFB8BABF), fontSize: 30.sp),
                    hintMaxLines: 1,
                    enabledBorder: new UnderlineInputBorder(
                      borderSide: BorderSide(color: Color(0x19000000)),
                    ),
                    focusedBorder: new UnderlineInputBorder(
                      borderSide: BorderSide(color: Colors.blueAccent),
                    ),
                  ),
                  style: TextStyle(
                    color: Colors.black,
                    fontSize: 32.sp,
                  ),
                  keyboardType: TextInputType.text,
                  onChanged: (value) {
                    this.videoFps = int.parse(value);
                  },
                ),
              ),
              getTextWidget("分辨率",
                  margin: EdgeInsets.only(
                      left: 0, top: 15.sp, right: 0, bottom: 0)),
              Expanded(
                child: Container(
                  padding:
                      EdgeInsets.only(top: 0, bottom: 0, left: 20, right: 20),
                  width: 1.sw,
                  color: Colors.white,
                  child: ListView.separated(
                    separatorBuilder: this.getSeparatorBuilder,
                    itemBuilder: this.getResolutionItem,
                    itemCount: resolutionList.length,
                  ),
                ),
              ),
            ],
          ),
        ));
  }
}
